Package: simode
Type: Package
Title: Statistical Inference for Systems of Ordinary Differential
        Equations using Separable Integral-Matching
Version: 1.2.2
Authors@R: c(
    person("Itai", "Dattner", email = "idattner@stat.haifa.ac.il", role = "aut"),
    person("Rami", "Yaari", email = "ramiyaari@gmail.com", role = c("aut", "cre")))
Description: Implements statistical inference for systems of ordinary differential equations,
  that uses the integral-matching criterion and takes advantage of the separability of parameters,
  in order to obtain initial parameter estimates for nonlinear least squares optimization. 
  Dattner & Yaari (2018) <arXiv:1807.04202>. 
  Dattner et al. (2017) <doi:10.1098/rsif.2016.0525>.
  Dattner & Klaassen (2015) <doi:10.1214/15-EJS1053>.
Depends: R (>= 3.4.0)
Imports: deSolve, pracma, quadprog, glmnet, ncvreg,
Suggests: parallel, knitr, testthat (>= 3.0.0), rmarkdown
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-02-16 20:32:54 UTC; ry2460
Author: Itai Dattner [aut],
  Rami Yaari [aut, cre]
Maintainer: Rami Yaari <ramiyaari@gmail.com>
Config/testthat/edition: 3
Repository: CRAN
Date/Publication: 2024-02-16 20:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 10:22:49 UTC; unix
