# Generated by roxygen2: do not edit by hand

S3method(confint,profile.simode)
S3method(plot,confint.simode)
S3method(plot,list.simode)
S3method(plot,profile.simode)
S3method(plot,simode)
S3method(print,confint.simode)
S3method(print,profile.simode)
S3method(print,simode)
S3method(print,summary.simode)
S3method(profile,simode)
S3method(summary,list.simode)
S3method(summary,simode)
export(plot_trace)
export(simode)
export(simode.control)
export(solve_ode)
importFrom(deSolve,ode)
importFrom(glmnet,cv.glmnet)
importFrom(glmnet,glmnet)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(ncvreg,cv.ncvreg)
importFrom(ncvreg,ncvreg)
importFrom(pracma,cumtrapz)
importFrom(pracma,errorbar)
importFrom(pracma,interp1)
importFrom(pracma,lsqlincon)
importFrom(pracma,trapz)
importFrom(stats,D)
importFrom(stats,approx)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,smooth.spline)
